@@ -4,14 +4,14 @@
 # Use of this source code is governed by a BSD-style license that can be
 # found in the LICENSE file.
 
-"""OSX specific tests."""
+"""MACOS specific tests."""
 
 import os
 import re
 import time
 
 import psutil
-from psutil import OSX
+from psutil import MACOS
 from psutil.tests import create_zombie_proc
 from psutil.tests import get_test_subprocess
 from psutil.tests import HAS_BATTERY
@@ -23,7 +23,7 @@ from psutil.tests import sh
 from psutil.tests import unittest
 
 
-PAGESIZE = os.sysconf("SC_PAGE_SIZE") if OSX else None
+PAGESIZE = os.sysconf("SC_PAGE_SIZE") if MACOS else None
 
 
 def sysctl(cmdline):
@@ -76,7 +76,7 @@ def human2bytes(s):
     return int(num * prefix[letter])
 
 
-@unittest.skipIf(not OSX, "OSX only")
+@unittest.skipIf(not MACOS, "MACOS only")
 class TestProcess(unittest.TestCase):
 
     @classmethod
@@ -101,7 +101,7 @@ class TestProcess(unittest.TestCase):
             time.strftime("%Y", time.localtime(start_psutil)))
 
 
-@unittest.skipIf(not OSX, "OSX only")
+@unittest.skipIf(not MACOS, "MACOS only")
 class TestZombieProcessAPIs(unittest.TestCase):
 
     @classmethod
@@ -158,11 +158,8 @@ class TestZombieProcessAPIs(unittest.TestCase):
         self.assertRaises((psutil.ZombieProcess, psutil.AccessDenied),
                           self.p.threads)
 
-    def test_memory_maps(self):
-        self.assertRaises(psutil.ZombieProcess, self.p.memory_maps)
 
-
-@unittest.skipIf(not OSX, "OSX only")
+@unittest.skipIf(not MACOS, "MACOS only")
 class TestSystemAPIs(unittest.TestCase):
 
     # --- disk
@@ -225,12 +222,6 @@ class TestSystemAPIs(unittest.TestCase):
         psutil_val = psutil.virtual_memory().free
         self.assertAlmostEqual(psutil_val, vmstat_val, delta=MEMORY_TOLERANCE)
 
-    @retry_before_failing()
-    def test_vmem_available(self):
-        vmstat_val = vm_stat("inactive") + vm_stat("free")
-        psutil_val = psutil.virtual_memory().available
-        self.assertAlmostEqual(psutil_val, vmstat_val, delta=MEMORY_TOLERANCE)
-
     @retry_before_failing()
     def test_vmem_active(self):
         vmstat_val = vm_stat("active")
@@ -291,7 +282,7 @@ class TestSystemAPIs(unittest.TestCase):
     @unittest.skipIf(not HAS_BATTERY, "no battery")
     def test_sensors_battery(self):
         out = sh("pmset -g batt")
-        percent = re.search("(\d+)%", out).group(1)
+        percent = re.search(r"(\d+)%", out).group(1)
         drawing_from = re.search("Now drawing from '([^']+)'", out).group(1)
         power_plugged = drawing_from == "AC Power"
         psutil_result = psutil.sensors_battery()
